/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.modger.message;

/**
 * An enumeration that defines message types
 * 
 * @author Cruz Bishop
 * @since 1.0.0.0
 */
public enum MessageType {
    
    /**
     * Trace-level data that is rarely needed except in deep debugging
     */
    TRACE(true),
    
    /**
     * Data used when debugging; This is not usually needed by a user
     */
    DEBUG(true),
    
    /**
     * Information that is not as important as regular output, but may be used by a normal user
     */
    INFORMATION(false),
    
    /**
     * Regular output; The equivalent of System.out.println
     */
    REGULAR(false),
    
    /**
     * A warning; Useful for users to know in case they must stop doing something
     */
    WARNING(false),
    
    /**
     * An error, required when something goes wrong
     */
    ERROR(false);
    
    /**
     * Creates a new message type
     * 
     * @param filtered True if filtered, otherwise false
     * @since 1.0.0.0
     */
    MessageType(boolean filtered) {
        this.filtered = filtered;
    }
    
    /**
     * The filtered status of this message type
     * 
     * @since 1.0.0.0
     */
    private boolean filtered;
    
    /**
     * Checks to see if this message type is filtered
     * 
     * @return The filtered status
     * @since 1.0.0.0
     */
    public boolean isFiltered() {
        return this.filtered;
    }
    
    /**
     * Filters this message type
     * 
     * @since 1.0.0.0
     */
    public void filter() {
        this.filtered = true;
    }
    
    /**
     * Unfilters this message type
     * 
     * @since 1.0.0.0
     */
    public void unfilter() {
        this.filtered = false;
    }
    
}
