/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.modger.message.handler.impl.external;

import biz.massivedynamics.modger.message.Message;
import biz.massivedynamics.modger.message.MessageType;
import biz.massivedynamics.modger.message.handler.MessageHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * A message handler that forwards log messages to java.util.logging.Logger
 *
 * @author Cruz Bishop
 * @since 1.1.0.0
 */
public class JDKLoggerMessageHandler extends MessageHandler {
    
    /**
     * Creates a new JDKLoggerMessageHandler
     * 
     * @since 1.1.0.0
     */
    public JDKLoggerMessageHandler() {}
    
    /**
     * Creates a new JDKLoggerMessageHandler
     * 
     * @param name The name to use
     * @since 1.1.0.0
     */
    public JDKLoggerMessageHandler(String name) {
        this.logger = Logger.getLogger(name);
    }
    
    /**
     * Creates a new JDKLoggerMessageHandler
     * 
     * @param logger The Modger logger to base this logger from
     * @since 1.1.0.0
     */
    public JDKLoggerMessageHandler(biz.massivedynamics.modger.Logger logger) {
        this.logger = Logger.getLogger(logger.getName());
    }
    
    /**
     * The logger to use
     * 
     * @since 1.1.0.0
     */
    private Logger logger;
    
    /**
     * Gets the logger to use
     * 
     * @return The logger
     * @since 1.1.0.0
     */
    public Logger getLogger() {
        return this.logger;
    }
    
    /**
     * Gets the {@link Level} associated with the specified {@link MessageType}
     * 
     * @param type The {@link MessageType} being used
     * @return The {@link Level} to use
     * @since 1.1.0.0
     */
    public static Level getLevel(MessageType type) {
        
        //Start switching
        switch (type) {
            //First, handle TRACE
            case TRACE:
                return Level.FINER;
                
            case DEBUG:
                return Level.FINE;
                
            case INFORMATION:
                return Level.INFO;
                
            case REGULAR:
                return Level.INFO;
                
            case WARNING:
                return Level.WARNING;
                
            case ERROR:
                return Level.SEVERE;
                
            default:
                return Level.INFO;
                
        }
        
    }

    /**
     * Submits the specified message
     * 
     * @param message The message
     * @throws Exception Any exceptions that occur
     * @since 1.1.0.0
     */
    @Override
    public void submit(Message message) throws Exception {
        if (this.logger == null) {
            this.logger = Logger.getLogger(message.getLogger().getName());
        }
        logger.logp(getLevel(message.getType()), logger.getName(), null, message.getRawMessage());
    }    

    /**
     * Submits the specified message with the attached cause
     * 
     * @param message The message
     * @param cause The cause
     * @throws Exception Any exceptions that occur
     */
    @Override
    public void submit(Message message, Throwable cause) throws Exception {
        if (this.logger == null) {
            this.logger = Logger.getLogger(message.getLogger().getName());
        }
        logger.logp(getLevel(message.getType()), logger.getName(), null, message.getRawMessage(), cause);
    }
    
    
    
}
