/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.modger.message.handler.impl.external;

import biz.massivedynamics.modger.message.Message;
import biz.massivedynamics.modger.message.MessageType;
import biz.massivedynamics.modger.message.handler.MessageHandler;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 * A {@link MessageHandler} that provides compatibility with Log4J
 * 
 * @author Cruz Bishop
 * @since 1.1.0.0
 */
public class Log4JMessageHandler extends MessageHandler {
    
    /**
     * The logger to use
     * 
     * @since 1.1.0.0
     */
    private Logger logger;
    
    /**
     * Gets the logger
     * 
     * @return The logger
     * @since 1.1.0.0
     */
    public Logger getLogger() {
        return this.logger;
    }
    
    /**
     * Checks to see if Log4J has been configured
     * 
     * @since 1.1.0.0
     */
    private static boolean configured = false;
    
    /**
     * Creates and configures the Log4J logger if needed
     * 
     * @since 1.1.0.0
     */
    public Log4JMessageHandler() {
        if (!configured) {
            BasicConfigurator.configure();
        } else configured = true;
    }
    
    /**
     * Creates and configures the Log4J logger if needed
     * 
     * @param logger The logger to use
     * @since 1.1.0.0
     */
    public Log4JMessageHandler(Logger logger) {
        if (!configured) {
            BasicConfigurator.configure();
        } else configured = true;
        this.logger = logger;
    }
    
    /**
     * Creates and configures the Log4J logger if needed
     * 
     * @param logger The Modger logger to use
     * @since 1.1.0.0
     */
    public Log4JMessageHandler(biz.massivedynamics.modger.Logger logger) {
        if (!configured) {
            BasicConfigurator.configure();
        } else configured = true;
        this.logger = Logger.getLogger(logger.getName());
    }
    
    /**
     * Gets the {@link Level} associated with the specified {@link MessageType}
     * 
     * @param type The {@link MessageType} to replace
     * @return The {@link Level} to be used
     * @since 1.1.0.0
     */
    public Level getLevel(MessageType type) {
        
        //Start switching by type
        switch (type) {
            
            case TRACE:
                return Level.TRACE;
                
            case DEBUG:
                return Level.DEBUG;
                
            case INFORMATION:
            case REGULAR:
                return Level.INFO;
                
            case WARNING:
                return Level.WARN;
                
            case ERROR:
                return Level.ERROR;
            
            //By default, it's information
            default:
                return Level.INFO;
        }
        
    }

    /**
     * Submits a {@link Message}
     * 
     * @param message The message to submit
     * @throws Exception Any exceptions that may occur
     * @since 1.1.0.0
     */
    @Override
    public void submit(Message message) throws Exception {
        if (this.logger == null) {
            this.logger = Logger.getLogger(message.getLogger().getName());
        }
        logger.log(this.getLevel(message.getType()), message.getRawMessage());
    }

    /**
     * Submits a {@link Message} with an {@link Exception} as the cause
     * 
     * @param message The message to submit
     * @param cause The cause
     * @throws Exception Any exceptions that may occur
     * @since 1.1.0.0
     */
    @Override
    public void submit(Message message, Throwable cause) throws Exception {
        if (this.logger == null) {
            this.logger = Logger.getLogger(message.getLogger().getName());
        }
        logger.log(this.getLevel(message.getType()), message.getRawMessage(), cause);
    }
    
}
